from pydantic import BaseModel
from typing import Literal, List, ClassVar
from ..utils.core import write_to_file
from .styles import TextStyles

class Text(BaseModel):
    content: str
    styles: str | TextStyles = ""  # CSS styles
    filepath: str = "./output.html"

    def write_to_file(self, method="a"):
        # Apply the style attribute directly to the div
        html_content = f'<div style="{self.styles if isinstance(self.styles, str) else self.styles.get()}">{self.content}</div>'
        write_to_file(html_content, self.filepath, method)

    def get_html(self, style_extra=""):
        html_content = f'<div style="{self.styles if isinstance(self.styles, str) else self.styles.get()}; {style_extra}">{self.content}</div>'
        return html_content



from pydantic import BaseModel
from ..utils.core import write_to_file
from .styles import ButtonStyles
from .styles import ButtonStylesTW

class Button(BaseModel):
    text: str
    onclick: str  # JavaScript function as a string
    filepath: str = './output.html'
    styles: ButtonStyles | str | ButtonStylesTW = ButtonStyles(backgroundColor="Primary", size="Medium").get()
    value: str = ""

    def write_to_file(self, method="a"):
        # Improved handling for HTML attributes with proper escaping
        if isinstance(self.styles, ButtonStylesTW):
            classes = self.styles.get()
        else:
            classes = ""
        
        value_attr = f"value='{self.value}'" if self.value else ""
        button_html = f'<button {classes}   {value_attr} style="{self.styles if isinstance(self.styles, str) else self.styles.get() if isinstance(self.styles, ButtonStyles) else ""}" onclick=\"{self.onclick}\">{self.text}</button>\n'
        write_to_file(button_html, self.filepath, method)

    def get_html(self, style_extra=""):
        value_attr = f"value='{self.value}'" if self.value else ""
        button_html = f'<button  {classes} {value_attr} style="{self.styles if isinstance(self.styles, str) else self.styles.get()}; {style_extra}" onclick="{self.onclick}">{self.text}</button>'
        return button_html



from pydantic import BaseModel
from typing import Any, Dict, Optional

class EventHandler(BaseModel):
    event: str
    action: str  # Changed from Callable to str to directly provide JavaScript code
    params: Dict[str, Any] = {}

    def get_alpine_directive(self) -> str:
        # Directly use action as the JavaScript code for Alpine.js
        return f"x-on:{self.event}=\"{self.action}\""



from pydantic import BaseModel

class ButtonAlert(BaseModel):
    alert_text: str

    def get(self):
        return f"alert('{self.alert_text}')"

class ButtonLink(BaseModel):
    url: str

    def get(self):
        # Correct the quoting for the JavaScript function
        return f"window.location.href='{self.url}';", "Redirect"

class ButtonModal(BaseModel):
    id: str

    def get(self):
        return f"document.getElementById('{self.id}').style.display='block';"



from pydantic import BaseModel
from typing import List, Dict, Union, Literal
from ..utils.core import write_to_file
from ..text.text import Text
from ..button.button import Button

class Grid(BaseModel):
    rows: int
    cols: int
    gap: int = 10  # Default gap between grid items
    items: List[Dict[str, Union[str, BaseModel]]] = []
    filepath: str = "./output.html"

    def add_item(self, item: BaseModel, row_start: int, col_start: int, row_end: int = None, col_end: int = None):
        self.items.append({
            "item": item,
            "position": (row_start, col_start, row_end, col_end)
        })

    def render(self):
        grid_style = f"display: grid; grid-template-rows: repeat({self.rows}, 1fr); grid-template-columns: repeat({self.cols}, 1fr); gap: {self.gap}px;"
        grid_items_html = ""
        for item in self.items:
            item_obj, position = item["item"], item["position"]
            item_style = f"grid-row: {position[0]} / {position[2] if position[2] else 'span 1'}; grid-column: {position[1]} / {position[3] if position[3] else 'span 1'};"
            item_html = item_obj.get_html(style_extra=item_style)
            grid_items_html += item_html

        return f'<div style="{grid_style}">{grid_items_html}</div>'

    def write_to_file(self, method="w"):
        grid_html = self.render()
        write_to_file(grid_html, self.filepath, method)



from text.text import Text
from text.styles import TextStyles

text_styles = TextStyles(variant=["Bold", "Underline"], fontSize=25, fontFamily="Sans")
# This will create a new HTML file with the basic structure and add the first content
text_example = Text(content="Hello, world!", styles=text_styles)
text_example.write_to_file(method="w")

# This will append new content to the existing HTML file
text_2 = Text(content="Another example text.")
text_2.write_to_file(method="a")



from pydantic import BaseModel
from typing import Union, Any
from ..utils import write_to_file
from ..text import TextStyles

class Modal(BaseModel):
    id: str  # Unique identifier for the modal
    title: str  # Title displayed on the modal
    title_style: Union[TextStyles, None] = None  # Optional TextStyles for the title
    content: Union[str, Any]  # Content could be a string or any class instance
    content_style: Union[TextStyles, None] = None  # Optional TextStyles for the content
    filepath: str = './output.html'

    def write_to_file(self, method="a"):
        modal_html = self.get_html()
        write_to_file(modal_html, self.filepath, method)

    def get_html(self, style_extra=""):
        title_style = self.title_style.get() if self.title_style else ""

        # Render content based on its type
        if hasattr(self.content, 'get_html'):
            content_html = self.content.get_html()  # Call the method if it's a class instance
        else:
            content_style = self.content_style.get() if self.content_style else ""
            content_html = f'<p style="{content_style}">{self.content}</p>'  # Treat as plain text otherwise

        return f"""
<div id="{self.id}" style="display:none; position: fixed; z-index: 1; left: 0; top: 0; width: 100%; height: 100%; overflow: auto; background-color: rgba(0,0,0,0.4); padding-top: 60px; border-radius: 15; {style_extra}">
    <div style="background-color: #fefefe; margin: 5% auto; padding: 20px; border: 1px solid #888; width: 80%; border-radius: 15px;">
        <div style="display: flex; justify-content: space-between; align-items: center; border-radius: 15;">
            <h2 style="{title_style}">{self.title}</h2>
            <button onclick="document.getElementById('{self.id}').style.display='none'" style="font-size: 25px; border: none; cursor: pointer;">&times;</button>
        </div>
        {content_html}
    </div>
</div>
<script>
function showModal(id) {{
    document.getElementById(id).style.display = 'block';
}}
</script>
        """



from pydantic import BaseModel
from typing import Union, Dict 

class Tailwind(BaseModel):
    tailwind_styles: str = ""


    def combine(self):
        return f""" class="{self.tailwind_styles}" """


    def get(self):
        # should be overridden with customizing the styles, but call the combine method
        pass 




from pydantic import BaseModel, BaseConfig
from typing import Literal, List, ClassVar, Dict


def write_to_file(content, filepath, method='a'):
    if method == 'w':
        # Creating a new HTML document with the content
        full_content = f"""<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Generated Page</title>
    <script defer src="https://cdn.jsdelivr.net/npm/alpinejs@3.x.x/dist/cdn.min.js"></script>
    <script src="https://cdn.tailwindcss.com"></script>
</head>
<body>
{content}
</body>
</html>"""
    else:
        # Appending content to an existing HTML document
        try:
            with open(filepath, 'r+') as file:
                file_content = file.read()
                insert_position = file_content.rfind('</body>')
                if insert_position != -1:
                    # Insert the new content just before the </body> tag
                    file_content = f"{file_content[:insert_position]}{content}\n{file_content[insert_position:]}"
                    file.seek(0)  # Go back to the start of the file
                    file.write(file_content)  # Rewrite with new content
                    file.truncate()  # Remove any remaining original content after the new end
                else:
                    raise ValueError("No </body> tag found in the existing HTML document.")
        except FileNotFoundError:
            print("File not found. Creating a new file.")
            method = 'w'  # Switch to write mode to create the file if it doesn't exist
            return write_to_file(content, filepath, method)

    # If creating a new file, this part is executed
    if method == 'w':
        with open(filepath, 'w') as file:
            file.write(full_content)




from button.button import Button, ButtonAlert, ButtonLink

onclick, value = ButtonLink(url="https://google.com").get()


button_example = Button(text="Hi", onclick=onclick, value=value)


button_example.write_to_file(method="w")



from pydantic import BaseModel
from typing import List, Optional, Union, Any
from ..utils import write_to_file
from ..text.styles import TextStyles

class CardAction(BaseModel):
    label: str
    onclick: str  # JavaScript function or URL for action

class Card(BaseModel):
    title: str
    title_style: Optional[TextStyles] = None  # Optional TextStyles for the title
    content: Union[str, Any]  # Allows for string content or any object with a get_html method
    actions: Optional[List[CardAction]] = None
    filepath: str = './output.html'

    def write_to_file(self, method="a"):
        card_html = self.get_html()
        write_to_file(card_html, self.filepath, method)

    def get_html(self, style_extra=""):
        # Apply title styles if provided
        title_style = self.title_style.get() if self.title_style else "font-size: 24px; font-weight: bold;"  # Default style for title

        # Create action buttons if any
        actions_html = ""
        if self.actions:
            actions_html = "".join([f'<button onclick="{action.onclick}">{action.label}</button>' for action in self.actions])

        # Check for content object with get_html method
        if hasattr(self.content, 'get_html'):
            content_html = self.content.get_html()
        else:
            content_html = self.content

        # Embedding enhanced CSS for the card
        styles = """
<style>
.card {
    box-shadow: 0 4px 8px 0 rgba(0,0,0,0.2);
    transition: 0.3s;
    border-radius: 10px; /* Rounded corners */
    background-color: #fff; /* White background */
    margin: 10px 0; /* Margin for spacing */
    width: auto; /* Adjust width as needed */
}
.card-container {
    padding: 16px;
}
.card-title {
    margin-bottom: 15px;
}
.card button {
    cursor: pointer;
    padding: 10px;
    border: none;
    border-radius: 5px;
    margin-right: 10px;
    background-color: #007bff;
    color: white;
}
.card button:hover {
    opacity: 0.8;
}
</style>
        """

        return f"""
<div class="card rounded-xl border bg-card text-card-foreground shadow" style="{style_extra}">
    <div class="card-container flex flex-col space-y-1.5 p-6">
        <h4 class="card-title font-semibold leading-none tracking-tight " style="{title_style}">{self.title}</h4>
        <p>{content_html}</p>
        <div>{actions_html}</div>
    </div>
</div>
        """



from pydantic import BaseModel
from typing import List, Literal, Optional, Dict
from ..tailwind import Tailwind

class ButtonStyles(BaseModel):
    size: Literal["Small", "Medium", "Large"] = "Medium"
    backgroundColor: Literal["Primary", "Secondary", "Success"] | str = "Primary"
    textColor: str = "#ffffff"  # Default to white text
    borderStyle: Literal["None", "Solid", "Dashed", "Dotted"] = "Solid"
    borderRadius: Literal["Rounded", "Rounded-XL", "Rounded-SM", "Rounded-MD", "Rounded-LG"] | int = "Rounded"
    fontWeight: Literal["Normal", "Bold"] = "Normal"
    fontSize: Optional[int] = None  # Allow users to optionally set font size

    def get(self) -> str:
        size_map = {
            "Small": "padding: 5px 10px; font-size: 12px;",
            "Medium": "padding: 10px 20px; font-size: 14px;",
            "Large": "padding: 15px 30px; font-size: 16px;",
        }

        background_color_map = {
            "Primary": "#007bff",
            "Secondary": "#6c757d",
            "Success": "#28a745",
        }

        border_radius_map = {
            "Rounded": "border-radius: 5px;",
            "Rounded-SM": "border-radius: 2px;",
            "Rounded-MD": "border-radius: 8px;",
            "Rounded-LG": "border-radius: 12px;",
            "Rounded-XL": "border-radius: 20px;",
        }

        background_color = background_color_map.get(self.backgroundColor, self.backgroundColor)
        border_radius = border_radius_map.get(self.borderRadius, f"border-radius: {self.borderRadius}px;") if isinstance(self.borderRadius, str) else f"border-radius: {self.borderRadius}px;"

        styles = f"""
            background-color: {background_color};
            color: {self.textColor};
            border: 2px {self.borderStyle.lower()} #dee2e6;
            {border_radius}
            font-weight: {self.fontWeight.lower()};
            {size_map[self.size]}
            cursor: pointer;
        """

        if self.fontSize:
            styles += f"font-size: {self.fontSize}px; "

        return " ".join(styles.split())  # Clean up whitespace for a tidier output


# Better for non-dynamic things...
class ButtonStylesTW(Tailwind):
    rounded: Literal["Sm", "Md", "Lg", "Xl"] = "Md"
    buttonColor: Dict[Literal["Gray", "Red", "Green", "Blue"], Literal[100, 300, 500, 900]] | str = {"Gray": 500}
    font: Literal["Sans", "Serif"] = "Sans"
    size: Literal["Sm", "Md", "Lg", "Xl"] = "Md"
    fontColor: Literal["Offwhite", "Offblack"] = "Offblack"
    #shape: Literal["Rectangle", "Square"] = "Rectangle"

    def get(self):
        roundedMapping = {
            "Sm": "rounded-sm",
            "Md": "rounded",
            "Lg": "rounded-lg",
            "Xl": "rounded-xl"
        }
        buttonColorNameMapping = {
            "Gray": "bg-gray-{number}",
            "Red": "bg-rose-{number}",
            "Green": "bg-green-{number}",
            "Blue": "bg-blue-{number}"
        }

        fontMapping = {
            "Sans": "font-sans",
            "Serif": "font-serif"
        }

        sizeMapping = {
            "Sm": "w-24 h-10",
            "Md": "w-36 h-16",
            "Lg": "w-48 h-28",
            "Xl": "w-60 h-32"
        }

        fontColor = "text-gray-200" if self.fontColor == "Offwhite" else "text-gray-900"
        tailwind = [fontColor]

        tailwind.append(roundedMapping.get(self.rounded, ""))
        tailwind.append(buttonColorNameMapping.get(list(self.buttonColor.keys())[0]).format(number=list(self.buttonColor.values())[0]))
        tailwind.append(fontMapping.get(self.font))
        tailwind.append(sizeMapping.get(self.size))

        output = " ".join(word for word in tailwind)
        self.tailwind_styles = output 
        print(self.combine())

        return self.combine()





from pydantic import BaseModel
from typing import Literal, List

class TextStyles(BaseModel):
    fontSize: int = 20
    variant: List[Literal["Bold", "Italics", "Underline"]] | None = None
    color: str = "#171717"  # hex code INCLUDING the hashtag
    fontFamily: Literal["Sans", "Serif"] = "Sans"

    def get(self) -> str:
        fontSize = f"font-size: {self.fontSize}px; "
        variant = ""
        color = f"color: {self.color}; "
        if self.fontFamily == "Sans":
            fontFamily = "font-family: sans-serif; "
        else:
            fontFamily = "font-family: serif; "

        if self.variant:
            for v in self.variant:
                if v == "Bold":
                    variant += "font-weight: bold; "
                elif v == "Italics":
                    variant += "font-style: italic; "  # Corrected to 'italic'
                elif v == "Underline":
                    variant += "text-decoration: underline; "

        return fontSize + color + fontFamily + variant

class TextDefaults(BaseModel):
    variant: Literal["Header", "Paragraph", "Subtitle"] # more to be implemented...

    def get(self) -> TextStyles:
        if self.variant == "Header":
            return TextStyles(fontSize=30, variant=["Bold"], fontFamily="Sans")
        elif self.variant == "Paragraph":
            return TextStyles(fontSize=20, fontFamily="Sans")
        elif self.variant == "Subtitle":
            return TextStyles(fontSize=25, fontFamily="Sans", variant=["Italics"], color="#333333")
        # more to be implemented... slowly but surely



from text import Text
from text import TextStyles, TextDefaults
from button import Button
from grid import Grid
from button import ButtonStyles,ButtonModal
from modal import Modal
from card import Card, CardAction

textstyles = TextStyles(variant=["Bold", "Underline"], fontSize=15, fontFamily="Sans")
header = TextDefaults(variant="Header").get()
buttonstyles = ButtonStyles(backgroundColor="Primary", size="Medium", borderRadius="Rounded-LG").get()


my_modal_text = Text(content="Hello, Modal!", styles=textstyles, filepath="./output.html"
)
my_modal = Modal(
    id="myModal",
    title="My Modal Title",
    title_style=header,
    content=my_modal_text,
    content_style=TextDefaults(variant="Paragraph").get()
)

# Create a grid
my_grid = Grid(rows=4, cols=4, gap=15)

# Create text and button components
my_text = Text(content="Hello, Grid!", styles=header, filepath="./output.html")
my_button = Button(text="Click Me", onclick="alert('Button clicked!')", filepath="./output.html", styles=buttonstyles)

button_2 = Button(text="Open", onclick=ButtonModal(id=my_modal.id).get(), filepath="./output.html", styles=buttonstyles)

# Define actions for the card
card_actions = [
    CardAction(label="Click Me", onclick="alert('Hello World')"),
    CardAction(label="Visit", onclick="window.location.href='https://example.com'")
]

# Create a card instance
my_card = Card(
    title="Card Title",
    title_style=header,
    content="This is the content of the card. It can be simple text or structured HTML.",
    actions=card_actions
)

my_card.write_to_file(method='w')


# Add components to the grid

my_grid.add_item(my_modal, row_start=1, col_start=1, row_end=4, col_end=4)
my_grid.add_item(my_card, row_start=3, col_start=5, row_end=4, col_end=4)
my_grid.add_item(button_2, row_start=1, col_start=1, row_end=2, col_end=2)



# Write the grid to file
my_grid.write_to_file(method='w')



from pydantic import BaseModel
from typing import Optional, Dict, Any
from ashrio import write_to_file, ButtonStyles  # Adjust the import path as necessary
from .core import EventHandler  # Adjust the import path as necessary
import json

class StatefulButton(BaseModel):
    initial_state: Dict[str, Any] # can include props, styles, and tailwind
    filepath: str = "./output.html"
    event_handlers: Optional[Dict[str, EventHandler]] = None

    def write_to_file(self, method="a"):
        # Serialize initial_state to a JSON string for Alpine.js x-data
        alpine_data = f"x-data='{json.dumps(self.initial_state)}'"
        # Generate all event directives from event_handlers
        event_directives = " ".join([handler.get_alpine_directive() for handler in self.event_handlers.values()]) if self.event_handlers else ""
        # Directly apply styles without dynamic binding
        button_html = f'<button {alpine_data} {event_directives} style="{self.initial_state.get("styles", "")}" x-text="text"></button>'
        write_to_file(button_html, self.filepath, method)

    def get_html(self):
        # Serialize initial_state to a JSON string for Alpine.js x-data
        alpine_data = f"x-data='{json.dumps(self.initial_state)}'"
        # Generate all event directives from event_handlers
        event_directives = " ".join([handler.get_alpine_directive() for handler in self.event_handlers.values()]) if self.event_handlers else ""
        # Directly apply styles without dynamic binding
        button_html = f'<button {alpine_data} {event_directives} class="{self.initial_state.get("tailwind", "")}" style="{self.initial_state.get("styles", "")}" x-text="text"></button>'
        return button_html


from pydantic import BaseModel
from typing import Optional, Dict, Any
from ashrio import write_to_file, TextStyles
from .core import EventHandler
import json  # Import the json module

class StatefulText(BaseModel):
    initial_state: Dict[str, Any]
    styles: str | TextStyles = ""
    filepath: str = "./output.html"
    event_handlers: Optional[Dict[str, EventHandler]] = None

    def write_to_file(self, method="a"):
        # Serialize initial_state to a JSON string for Alpine.js x-data
        alpine_data = f"x-data='{{\"text\": \"{self.initial_state['text']}\"}}'"
        # Generate all event directives from event_handlers
        event_directives = " ".join([handler.get_alpine_directive() for handler in self.event_handlers.values()]) if self.event_handlers else ""
        # Include x-text to bind the div's content to the Alpine.js data model's text property
        html_content = f'<div {alpine_data} {event_directives} style="{self.styles if isinstance(self.styles, str) else self.styles.get()}" x-text="text"></div>'
        write_to_file(html_content, self.filepath, method)

    def get_html(self, style_extra=""):
        alpine_data = f"x-data='{{\"text\": \"{self.initial_state['text']}\"}}'"
        event_directives = " ".join([handler.get_alpine_directive() for handler in self.event_handlers.values()]) if self.event_handlers else ""
        html_content = f'<div {alpine_data} {event_directives} style="{self.styles if isinstance(self.styles, str) else self.styles.get()}; {style_extra}" x-text="text"></div>'
        return html_content
